
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripList(name, exclusive)
{
    this.id = -1;
    this.name = "";
    this.obj = {};
    this.length = 0;
    this.exclusive = false;

    this.getId = function()
    {
        return this.id;
    }

    this.setName = function(value)
    {
        if (value) this.name = "" + value;
    }

    this.setExclusive = function(value)
    {
        this.exclusive = "" + value == "true";
    }

    this.getExclusive = function()
    {
        return this.length > 0 && this.exclusive;
    }

    this.add = function(o, purge)
    {
        if (!o) return;
        var oldObj = this.obj[o.getKey()];
        if (oldObj)
        {
            oldObj.merge(o);
            if (purge) oldObj.purge();
        }
        else
        {
            this.length++;
            if (purge) o.purge();
            this.obj[o.getKey()] = o;
        }
    }

    this.remove = function(value)
    {
        if (!value || !this.obj[value]) return;
        delete this.obj[value];
        this.length--;
    }

    this.reset = function()
    {
        this.obj = {};
        this.length = 0;
    }

    this.sort = function(purge)
    {
        var a = [];
        for (var k in this.obj) a.push(this.obj[k]);
        a.sort(this.sortBy);
        this.reset();
        for (var i = 0; i < a.length; i++) this.add(a[i], purge);
    }

    this.clone = function(purge)
    {
        var tmp = new this.constructor(this.name, this.exclusive);
        for (var k in this) if (this[k] && typeof(this[k]) != "object" && !/^(name|length|exclusive)$/.test(k)) tmp[k] = this[k];
        for (var k in this.obj) tmp.add(this.obj[k].clone(purge));
        return tmp;
    }

    this.merge = function(o)
    {
        this.setExclusive(this.getExclusive() || o.getExclusive());
        for (var k in o.obj) this.add(o.obj[k].clone());
    }

    this.generateXml = function()
    {
        var tmp = "";
        for (var k in this.obj) tmp += this.obj[k].generateXml();
        if (tmp != "") return "\t\t" +
            "<" + this.name +
                (this.exclusive ? " exclusive=\"" + this.exclusive + "\"" : "") +
            ">\n" + tmp + "\t\t</" + this.name + ">\n";
        else return "";
    }

    this.sortBy = function(a, b)
    {
        if (a.getKey() < b.getKey()) return -1;
        if (a.getKey() > b.getKey()) return 1;
        return 0;
    }

    this.setName(name);
    this.setExclusive(exclusive);
}

function YaripElementList()
{
    this.id = -1;
    this.name = "";
    this.obj = {};
    this.length = 0;
}
YaripElementList.prototype = new YaripList;
YaripElementList.prototype.reduce = function()
{
    var a = [];
    for (var k in this.obj) a.push(this.obj[k]);
    this.reset();

    var r = [];
    var t = [];
    var o = a.shift();
    while (o)
    {
        r.push(o);
//        var noCheck = !/^\/?(\/([a-z][\w]*|\*)(\[@[a-z][\w]*='[^']*'(\ and\ )?\])*)+$/i.test(o.getXPath());
        var noCheck = !/^\/?(\/[a-z]\w*(\[(\d+|@[a-z]\w*='[^']*?'(\sand\s)?)+\])*)+$/i.test(o.getXPath());
        for (var i = 0; i < a.length; i++)
        {
            if (noCheck                                                // do not check
                || !/^\/?(\/[a-z]\w*(\[(\d+|@[a-z]\w*='[^']*?'(\sand\s)?)+\])*)+$/i.test(a[i].getXPath()) // wasn't auto generated
                || a[i].getXPath().indexOf(o.getXPath()) != 0          // does not contain
                || a[i].getXPath().charAt(o.getXPath().length) != "/"  // is not the end of a sub-path
               ) t.push(a[i]);
        }
        a = t;
        t = [];
        o = a.shift();
    }
    a = r;

    for (var i = 0; i < a.length; i++) this.add(a[i]);
}

function YaripElementWhitelist()
{
    this.id = 0;
    this.name = "elementWhitelist";
    this.obj = {};
    this.length = 0;
    this.exclusive = false;
}
YaripElementWhitelist.prototype = new YaripElementList;
YaripElementWhitelist.prototype.set = function(row, col, value)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ == row)
        {
            var o = this.obj[k];
            switch (col)
            {
                case 0:
                    if (!yarip.checkXPath(value)) break;

                    var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
                    fh.removeEntry("xpath", k);

                    o.setXPath(value);
                    this.remove(k);
                    this.add(o);

                    fh.addEntry("xpath", value);

                    this.sort();
                    break;
                case 1: o.setDeep(value); break;
//                case 2: o.setFound(value); break;
//                case 3: o.setNotFound(value); break;
//                case 4: o.setIgnored(value); break;
            }
        }
    }
}
YaripElementWhitelist.prototype.get = function(row, col)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ == row)
        {
            var o = this.obj[k];
            switch (col)
            {
                case 0: return k;
                case 1: return o.getDeep();
                case 2: return o.getFound();
                case 3: return o.getNotFound();
                case 4: return o.getIgnored();
                default: return "";
            }
        }
    }
    return "";
}
//YaripElementWhitelist.prototype.toArray = function()
//{
//    var a = [];
//    for (var k in this.obj)
//    {
//        var o = this.obj[k];
//        a[a.length] = [k, o.getDeep(), o.getFound(), o.getNotFound(), o.getIgnored()];
//    }
//    return a;
//}

function YaripElementBlacklist()
{
    this.id = 1;
    this.name = "elementBlacklist";
    this.obj = {};
    this.length = 0;
}
YaripElementBlacklist.prototype = new YaripElementList;
YaripElementBlacklist.prototype.set = function(row, col, value)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ == row)
        {
            var o = this.obj[k];
            switch (col)
            {
                case 0:
                    if (!yarip.checkXPath(value)) break;

                    var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
                    fh.removeEntry("xpath", k);

                    o.setXPath(value);
                    this.remove(k);
                    this.add(o);

                    fh.addEntry("xpath", value);

                    this.sort();
                    break;
                case 1: o.setDeep(value); break;
                case 2: o.setForce(value); break;
                case 3: o.setDOW(value); break;
            }
        }
    }
}
YaripElementBlacklist.prototype.get = function(row, col)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ == row)
        {
            var o = this.obj[k];
            switch (col)
            {
                case 0: return k;
                case 1: return o.getDeep();
                case 2: return o.getForce();
                case 3: return o.getDOW();
                case 4: return o.getFound();
                case 5: return o.getNotFound();
                case 6: return o.getIgnored();
                default: return "";
            }
        }
    }
    return "";
}
//YaripElementBlacklist.prototype.toArray = function()
//{
//    var a = [];
//    for (var k in this.obj)
//    {
//        var o = this.obj[k];
//        a[a.length] = [k, o.getDeep(), o.getForce(), o.getDOW(), o.getFound(), o.getNotFound(), o.getIgnored()];
//    }
//    return a;
//}
YaripElementBlacklist.prototype.generateCSS = function(notDeep)
{
    var tmp = "";
    for (var k in this.obj) tmp += this.obj[k].generateCSS(notDeep);
    if (tmp != "") return tmp.replace(/,(\n)$/, "$1") + "\t{\n\t\tdisplay: none !important;\n\t}\n";
    else return "";
}

function YaripContentWhitelist()
{
    this.id = 2;
    this.name = "contentWhitelist";
    this.obj = {};
    this.length = 0;
    this.exclusive = false;
}
YaripContentWhitelist.prototype = new YaripList;
YaripContentWhitelist.prototype.set = function(row, col, value)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ == row)
        {
            var o = this.obj[k];
            switch (col)
            {
                case 0:
                    if (!yarip.checkRegExp(value)) break;

                    var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
                    fh.removeEntry("regexp", k);

                    o.setRegExp(value);
                    this.remove(k);
                    this.add(o);

                    fh.addEntry("regexp", value);

                    this.sort();
                    break;
                case 1: o.setDeep(value); break;
//                case 2: o.setFound(value); break;
            }
        }
    }
}
YaripContentWhitelist.prototype.get = function(row, col)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ == row)
        {
            var o = this.obj[k];
            switch (col)
            {
//                case 0: return o.getType();
//                case 0: return o.getRegExp();
                case 0: return k;
                case 1: return o.getDeep();
                case 2: return o.getFound();
                default: return "";
            }
        }
    }
    return "";
}
//YaripContentWhitelist.prototype.toArray = function()
//{
//    var a = [];
//    for (var k in this.obj)
//    {
//        var o = this.obj[k];
//        a[a.length] = [/*o.getType(),*/ o.getRegExp(), o.getDeep(), o.getFound()];
//    }
//    return a;
//}

function YaripContentBlacklist()
{
    this.id = 3;
    this.name = "contentBlacklist";
    this.obj = {};
    this.length = 0;
}
YaripContentBlacklist.prototype = new YaripList;
YaripContentBlacklist.prototype.set = function(row, col, value)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ == row)
        {
            var o = this.obj[k];
            switch (col)
            {
                case 0:
                    if (!yarip.checkRegExp(value)) break;

                    var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
                    fh.removeEntry("regexp", k);

                    o.setRegExp(value);
                    this.remove(k);
                    this.add(o);

                    fh.addEntry("regexp", value);

                    this.sort();
                    break;
                case 1: o.setDeep(value); break;
                case 2: o.setForce(value); break;
//                case 3: o.setFound(value); break;
//                case 4: o.setIgnored(value); break;
            }
        }
    }
}
YaripContentBlacklist.prototype.get = function(row, col)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ == row)
        {
            var o = this.obj[k];
            switch (col)
            {
//                case 0: return o.getType();
//                case 0: return o.getRegExp();
                case 0: return k;
                case 1: return o.getDeep();
                case 2: return o.getForce();
                case 3: return o.getFound();
                case 4: return o.getIgnored();
                default: return "";
            }
        }
    }
    return "";
}
//YaripContentBlacklist.prototype.toArray = function()
//{
//    var a = [];
//    for (var k in this.obj)
//    {
//        var o = this.obj[k];
//        a[a.length] = [/*o.getType(),*/ o.getRegExp(), o.getDeep(), o.getForce(), o.getFound(), o.getIgnored()];
//    }
//    return a;
//}

function YaripStylesList()
{
    this.id = 4;
    this.name = "styles";
    this.obj = {};
    this.length = 0;
}
YaripStylesList.prototype = new YaripList;
YaripStylesList.prototype.set = function(row, col, value)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ == row)
        {
            var o = this.obj[k];
            switch (col)
            {
                case 0:
                    if (!yarip.checkXPath(value)) break;

                    var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
                    fh.removeEntry("xpath", k);

                    o.setXPath(value);
                    this.remove(k);
                    this.add(o);

                    fh.addEntry("xpath", value);

                    this.sort();
                    break;
                case 1:
                    var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
                    fh.removeEntry("style", o.getStyle());

                    o.setStyle(value);

                    fh.addEntry("style", value);
                    break;
                case 2: o.setDeep(value, true); break;
//                case 3: o.setFound(value); break;
//                case 4: o.setNotFound(value); break;
            }
        }
    }
}
YaripStylesList.prototype.get = function(row, col)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ == row)
        {
            var o = this.obj[k];
            switch (col)
            {
                case 0: return k;
                case 1: return o.getStyle();
                case 2: return o.getDeep();
                case 3: return o.getFound();
                case 4: return o.getNotFound();
                default: return "";
            }
        }
    }
    return "";
}
//YaripStylesList.prototype.toArray = function()
//{
//    var a = [];
//    for (var k in this.obj)
//    {
//        var o = this.obj[k];
//        a[a.length] = [k, o.style, o.getDeep(), o.getFound(), o.getNotFound()];
//    }
//    return a;
//}
YaripStylesList.prototype.generateCSS = function(notDeep)
{
    var tmp = "";
    for (var k in this.obj)
    {
//        if (!this.obj[k].getForce()) continue;
        if (!notDeep != this.obj[k].getDeep() || !this.obj[k].getForce()) continue;
        var xpathAsStyle = this.obj[k].xpathToStyle();
        if (xpathAsStyle) tmp += "\t" + xpathAsStyle + "\n\t{\n\t\t" + this.obj[k].style.replace(/\s*;(\s*\*\/)?\s*/g, ";$1\n\t\t").replace(/\n\t\t$/, "") + "\n\t}\n";
    }
    return tmp;
}

function YaripExtList()
{
    this.id = 5;
    this.name = "extension";
    this.obj = {};
    this.length = 0;
}
YaripExtList.prototype = new YaripList;
YaripExtList.prototype.set = function(row, col, value)
{
}
//YaripExtList.prototype.get = function(row, col)
//{
//    var i = 0;
//    for (var k in this.obj)
//    {
//        if (i++ == row)
//        {
//            var p = yarip.map.objId[k];
//            switch (col)
//            {
//                case 0: return p.getName();
//                default: return "";
//            }
//        }
//    }
//    return "";
//}
YaripExtList.prototype.get = function(row, col)
{
    var tmpArr = this.toArray(); // id array
    if (tmpArr <= row) return "";

    var pageArr = []; // page array
    for (var i = 0; i < tmpArr.length; i++)
    {
        var idArr = tmpArr[i];
        var page = yarip.map.objId[idArr[0]];
        if (page) pageArr.push(page);
        else
        {
            this.remove(idArr[0]);
            yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
        }
    }
    if (pageArr.length > row)
    {
        pageArr.sort(yarip.map.sortByPage);
        return pageArr[row].getName();
    }
    return "";
}
YaripExtList.prototype.toArray = function()
{
    var a = [];
    for (var k in this.obj) a[a.length] = [k];
    return a;
}
