
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripMonitorDialog()
{
    this.gBrowser = null;
    this.stringbundle = null;
    this.tree = null;
    this.doScroll = true;

    this.openAddress = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        var contentLocation = item.contentLocation;
        if (!contentLocation) return;

        if (this.gBrowser) this.gBrowser.selectedTab = this.gBrowser.addTab(contentLocation);
    }

    this.copyAddress = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        var contentLocation = item.contentLocation;
        if (!contentLocation) return;

        var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
        clipboardHelper.copyString(contentLocation);
    }

    this.whitelistContent = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
//        var address = item.address;
        var address = yarip.getAddress(item.location);
        if (!address) return;

        var regExp = yarip.generateRegExp(item.contentLocation);
        if (!regExp) return;

        var obj = {
            content: new YaripContent(null, regExp),
            pageAddress: address
        }

//        window.openDialog("chrome://yarip/content/whitelistcontentdialog.xul", "whitelistcontentdialog", "chrome,modal,resizable", item.doc, obj);
        window.openDialog("chrome://yarip/content/whitelistcontentdialog.xul", "whitelistcontentdialog", "chrome,modal,resizable", item.location, obj);

        if (!obj.content) return;
        if (!obj.pageAddress || obj.pageAddress == "") return;

        if (yarip.whitelistContent(obj.pageAddress, obj.content.getRegExp(), obj.content))
        {
            yarip.map.sort(false);
            yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
            yarip.pageDialog.reloadPage(obj.pageAddress);
        }
    }

    this.blacklistContent = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
//        var address = item.address;
        var address = yarip.getAddress(item.location);
        if (!address) return;

        var regExp = yarip.generateRegExp(item.contentLocation);
        if (!regExp) return;

        var obj = {
            content: new YaripContent(null, regExp),
            pageAddress: address
        }

//        window.openDialog("chrome://yarip/content/blacklistcontentdialog.xul", "blacklistcontentdialog", "chrome,modal,resizable", item.doc, obj);
        window.openDialog("chrome://yarip/content/blacklistcontentdialog.xul", "blacklistcontentdialog", "chrome,modal,resizable", item.location, obj);

        if (!obj.content) return;
        if (!obj.pageAddress || obj.pageAddress == "") return;

        if (yarip.blacklistContent(obj.pageAddress, obj.content.getRegExp(), obj.content))
        {
            yarip.map.sort(false);
            yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
            yarip.pageDialog.reloadPage(obj.pageAddress);
        }
    }

    this.copyRegExp = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        var regExp = yarip.generateRegExp(item.contentLocation);
        if (!regExp) return;

        var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
        clipboardHelper.copyString(regExp);
    }

    this.logContent = function(status, location, mode, contentLocation)
    {
        var item = {}
        item.location = {}
        item.location.href = location.href;
        item.location.protocol = location.protocol;
        item.location.host = location.host;
        item.location.pathname = location.pathname;
        item.mode = mode;
//        item.time = new Date().toLocaleString();
        var date = new Date();
        var hours = date.getHours();
        var minutes = date.getMinutes();
        var seconds = date.getSeconds();
        item.time = (hours < 10 ? "0" : "") + hours + ":" + (minutes < 10 ? "0" : "") + minutes + ":" + (seconds < 10 ? "0" : "") + seconds;
        item.status = status;
        item.page = location.href;
        item.contentLocation = contentLocation;
        yaripMonitorTreeView.appendItem(item, this.doScroll);
    }

    this.clear = function()
    {
        yaripMonitorTreeView.clearItems();
    }

    this.handleEvent = function(event)
    {
        if (!event) return;

        switch(event.target.id)
        {
            case "yarip-content-tree":
                this.doScroll = yaripMonitorTreeView.rowCount == this.tree.currentIndex + 1;
//                this.doScroll = yaripMonitorTreeView.rowCount > this.tree.currentIndex;
        }
    }

    this.load = function()
    {
        this.stringbundle = document.getElementById("monitor-dialog-stringbundle");
        this.tree = document.getElementById("yarip-content-tree");
        this.tree.view = yaripMonitorTreeView;
        this.tree.addEventListener("select", this, false);
    }

    this.unload = function()
    {
        this.hide();
    }

    this.id = -1;

    this.hidden = true;

    this.show = function(gBrowser)
    {
        if (this.hidden)
        {
            this.gBrowser = gBrowser;
            document.getElementById('yarip-monitor-splitter').hidden = false;
            document.getElementById('yarip-monitor').hidden = false;
//            yarip.monitorDialog = this;
            yarip.addMonitorDialog(this);
            this.hidden = false;
        }
    }

    this.hide = function()
    {
        if (!this.hidden)
        {
            document.getElementById('yarip-monitor-splitter').hidden = true;
            document.getElementById('yarip-monitor').hidden = true;
//            yarip.monitorDialog = null;
            yarip.removeMonitorDialog(this);
            this.hidden = true;
        }
    }
}

var yaripMonitorTreeView = {
    treebox: null,
    childData: [],
    whitelisted: Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService).getAtom("whitelisted"),
    blacklisted: Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService).getAtom("blacklisted"),

    get rowCount() { return this.childData.length; },

    cycleHeader: function(col) {},
    getCellProperties: function(row, col, properties)
    {
        if (col.id != "status") return;
        if (row < 0 || row >= this.childData.length) return;

        switch (this.childData[row].status)
        {
            case 1: properties.AppendElement(this.whitelisted); break;
            case 2: properties.AppendElement(this.blacklisted); break;
        }
    },
    getCellText: function(row, col)
    {
        var colid = col.id;
        if (colid == "status") return "";
        if (row < 0 || row >= this.childData.length) return "";

        var item = this.childData[row];
        return (colid in item ? item[colid] : null);
    },
    canDrop: function(index, orientation) { return false; },
    cycleCell: function(row, col) {},
    drop: function(row, orientation) { return 0; },
    getCellValue: function(row, col) { return ""; },
    getColumnProperties: function(col, properties) {},
    getImageSrc: function(row, col) { return ""; },
    getLevel: function(index) { return 0; },
    getParentIndex: function(rowIndex) { return -1; },
    getProgressMode: function(row, col) { return false; },
    getRowProperties: function(index, properties) {},
    hasNextSibling: function(rowIndex, afterIndex) { return false; },
//    hasNextSibling: function(rowIndex, afterIndex) { return rowIndex < afterIndex && this.childData.length >= afterIndex; },
    isContainerEmpty: function(index) { return false; },
    isContainer: function(index) { return false; },
    isContainerOpen: function(index) { return false; },
    isEditable: function(row, col) { return false; },
    isSelectable: function(row, col) { return false; },
    isSeparator: function(index) { return false; },
    isSorted: function() { return false; },
    performAction: function(action) {},
    performActionOnCell: function(action, row, col) {},
    performActionOnRow: function(action, row) {},
    selectionChanged: function() {},
    setCellText: function(row, col, value) {},
    setCellValue: function(row, col, value) {},
    setTree: function(treebox) { this.treebox = treebox; },
    toggleOpenState: function(index) {},

    appendItem: function(item, doScroll)
    {
        this.childData.push(item);
//        var index = this.visibleData.length;
//        this.visibleData.splice(index, 0, item);
//        this.treebox.rowCountChanged(index, 1);
        var index = this.childData.length - 1;
        this.treebox.rowCountChanged(index, 1);
        if (doScroll) this.treebox.ensureRowIsVisible(index);
    },

    getItem: function(index)
    {
        return this.childData[index];
    },

    clearItems: function()
    {
        var oldRows = this.rowCount;
        this.childData = [];
        this.treebox.rowCountChanged(0, -oldRows);
    }
}
