
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripPage(id, name, elementWhitelist, elementBlacklist, contentWhitelist, contentBlacklist, styles, extension)
{
    this.id = null;
    this.name = "";
    this.elementWhitelist = elementWhitelist ? elementWhitelist : new YaripElementWhitelist();
    this.elementBlacklist = elementBlacklist ? elementBlacklist : new YaripElementBlacklist();
    this.contentWhitelist = contentWhitelist ? contentWhitelist : new YaripContentWhitelist();
    this.contentBlacklist = contentBlacklist ? contentBlacklist : new YaripContentBlacklist();
    this.styles = styles ? styles : new YaripStylesList();
    this.extension = extension ? extension : new YaripExtList();

    this.setId = function(value)
    {
        if (value) this.id = "" + value;
    }

    this.getId = function()
    {
        return this.id;
    }

    this.setName = function(value)
    {
        if (value) this.name = "" + value;
    }

    this.getName = function()
    {
        return this.name;
    }

    this.clone = function(purge)
    {
        var tmp = new this.constructor(
            purge ? null : this.id,
            this.name,
            this.elementWhitelist.clone(purge),
            this.elementBlacklist.clone(purge),
            this.contentWhitelist.clone(purge),
            this.contentBlacklist.clone(purge),
            this.styles.clone(purge),
            this.extension.clone());
        for (var k in this) if (this[k] && typeof(this[k]) != "object" && !/^(id|name)$/.test(k)) tmp[k] = this[k];
        return tmp;
    }

    this.merge = function(page)
    {
        if (!page) return;
//        for (var v in page.elementWhitelist.obj) this.elementWhitelist.add(page.elementWhitelist.obj[v].clone());
//        for (var v in page.elementBlacklist.obj) this.elementBlacklist.add(page.elementBlacklist.obj[v].clone());
//        for (var v in page.contentWhitelist.obj) this.contentWhitelist.add(page.contentWhitelist.obj[v].clone());
////        this.contentWhitelist.setExclusive(this.contentWhitelist.getExclusive() || page.contentWhitelist.getExclusive());
//        for (var v in page.contentBlacklist.obj) this.contentBlacklist.add(page.contentBlacklist.obj[v].clone());
//        for (var v in page.styles.obj) this.styles.add(page.styles.obj[v].clone());
//        for (var v in page.extension.obj) this.extension.add(page.extension.obj[v].clone());
        this.elementWhitelist.merge(page.elementWhitelist);
        this.elementBlacklist.merge(page.elementBlacklist);
        this.contentWhitelist.merge(page.contentWhitelist);
        this.contentBlacklist.merge(page.contentBlacklist);
        this.styles.merge(page.styles);
        this.extension.merge(page.extension);
    }

    this.generateXml = function()
    {
        var tmp =
            this.elementWhitelist.generateXml() +
            this.elementBlacklist.generateXml() +
            this.contentWhitelist.generateXml() +
            this.contentBlacklist.generateXml() +
            this.styles.generateXml() +
            this.extension.generateXml();
        if (tmp != "") return "\t" +
            "<page id=\"" + this.id + "\" name=\"" + this.name + "\">\n" + tmp + "\t</page>\n";
        else return "";
    }

//    this.OFFICIAL_SCHEME = "(aaas|aaa|acap|afs|cap|cid|crid|data|dav|dict|dns|dtn|fax|file|ftp|gopher|go|h323|https|http|iax2|icap|imap|im|info|ipp|iris\\.beep|iris\\.lwz|iris\\.xpcs|iris\\.xpc|iris|ldap|mailserver|mailto|mid|modem|msrps|msrp|mtqp|mupdate|news|nfs|nntp|opaquelocktoken|pack|pop|pres|prospero|rtsp|service|shttp|sips|sip|snmp|soap\\.beeps|soap\\.beep|tag|telnet|tel|tftp|thismessage|tip|tn3270|tv|urn|vemmi|wais|xmlrpc\\.beeps|xmlrpc\\.beep|xmpp|z39\\.50r|z39\\.50s)";
//    this.UNOFFICIAL_SCHEME = "(about|addbook|afp|aim|applescript|bcp|bk|btspp|callto|castanet|cdv|chrome|chttp|cvs|daytime|device|doi|ed2k|eid|enp|feed|finger|fish|freenet|gg|gizmoproject|gsiftp|gsm-sms|h324|hdl|hnews|httpsy|iioploc|ilu|IOR|ircs|irc|itms|jar|javascript|jdbc|klik|kn|lastfm|ldaps|lifn|livescript|lrq|magnet|mailbox|man|md5|mms|mocha|moz-abmdbdirectory|msnim|myim|notes|nsfw|oai|pcast|phone|php|pop3|printer|psyc|pyimp|rdar|res|rsync|rvp|rwhois|rx|sdp|secondlife|sftp|skype|smb|sms|snews|soap\\.udp|soldat|ssh|steam|SubEthaEdit|svn\\+ssh|svn|t120|tann|tcp|telephone|txmt|uddi|unreal|ut2004|uuid|videotex|view-source|wcap|webcal|whodp|whois\\+\\+|whois|wpn|wtai|xeerkat|xfire|ymsgr)";
//    this.SCHEME = "(" + this.OFFICIAL_SCHEME + "|" + this.UNOFFICIAL_SCHEME + "):";
    this.SCHEME = "(\\w+):";
    this.IP_LITERAL = "(\\[((([a-f\\d]{1,4}:){6}([a-f\\d]{1,4}:[a-f\\d]{1,4})|((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])))|(::([a-f\\d]{1,4}:){5}([a-f\\d]{1,4}:[a-f\\d]{1,4})|((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])))|(([a-f\\d]{1,4})?::([a-f\\d]{1,4}:){4}([a-f\\d]{1,4}:[a-f\\d]{1,4})|((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])))|((([a-f\\d]{1,4}:)?[a-f\\d]{1,4})?::([a-f\\d]{1,4}:){3}([a-f\\d]{1,4}:[a-f\\d]{1,4})|((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])))|(((([a-f\\d]{1,4}:){1,2})?[a-f\\d]{1,4})?::([a-f\\d]{1,4}:){2}([a-f\\d]{1,4}:[a-f\\d]{1,4})|((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])))|(((([a-f\\d]{1,4}:){1,3})?[a-f\\d]{1,4})?::([a-f\\d]{1,4}:[a-f\\d]{1,4})|((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])))|(((([a-f\\d]{1,4}:){1,4})?[a-f\\d]{1,4})?::([a-f\\d]{1,4}:[a-f\\d]{1,4})|((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])))|(((([a-f\\d]{1,4}:){1,5})?[a-f\\d]{1,4})?::[a-f\\d]{1,4})|(((([a-f\\d]{1,4}:){1,6})?[a-f\\d]{1,4})?::))\\])";
    this.IPV4ADDRESS = "((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5]))";
    this.IP = "(" + this.IPV4ADDRESS + "|" + this.IP_LITERAL + ")";
//    this.AUTHORITY = "([a-z\\d]([\\w-.~!$&*+,=:]|%[a-f\\d]{2})*@)?";
//    this.REG_NAME = "([a-z\\d]([\\w-~!$&*+,=@]|%[a-f\\d]{2})+\\.)+";
    this.SIMPLE = "([a-z\\d]([\\w-~!$&'()*+,;=]|%[a-f\\d]{2})+)";
    this.ULD = "[a-z\\d]([\\w-.~!$&'()*+,;=]|%[a-f\\d]{2})+\\.";
//    this.SLD = this.SIMPLE + "\\.";
    this.TLD = "(ac|ad|aero|ae|af|ag|ai|al|am|an|ao|aq|arpa|ar|asia|as|at|au|aw|ax|az|ba|bb|bd|be|bf|bg|bh|biz|bi|bj|bm|bn|bo|br|bs|bt|bv|bw|by|bz|cat|ca|cc|cd|cf|cg|ch|ci|ck|cl|cm|cn|coop|com|co|cr|cu|cv|cx|cy|cz|de|dj|dk|dm|do|dz|ec|edu|ee|eg|er|es|et|eu|fi|fj|fk|fm|fo|fr|ga|gb|gd|ge|gf|gg|gh|gi|gl|gm|gn|gov|gp|gq|gr|gs|gt|gu|gw|gy|hk|hm|hn|hr|ht|hu|id|ie|il|im|info|int|in|io|iq|ir|is|it|je|jm|jobs|jo|jp|ke|kg|kh|ki|km|kn|kp|kr|kw|ky|kz|la|lb|lc|li|lk|lr|ls|lt|lu|lv|ly|ma|mc|md|me|mg|mh|mil|mk|ml|mm|mn|mobi|mo|mp|mq|mr|ms|mt|museum|mu|mv|mw|mx|my|mz|name|na|nc|net|ne|nf|ng|ni|nl|no|np|nr|nu|nz|om|org|pa|pe|pf|pg|ph|pk|pl|pm|pn|pro|pr|ps|pt|pw|py|qa|re|ro|rs|ru|rw|sa|sb|sc|sd|se|sg|sh|si|sj|sk|sl|sm|sn|so|sr|st|su|sv|sy|sz|tc|td|tel|tf|tg|th|tj|tk|tl|tm|tn|to|tp|travel|tr|tt|tv|tw|tz|ua|ug|uk|um|us|uy|uz|va|vc|ve|vg|vi|vn|vu|wf|ws|xn--0zwm56d|xn--11b5bs3a9aj6g|xn--80akhbyknj4f|xn--9t4b11yi5a|xn--deba0ad|xn--g6w251d|xn--hgbk6aj7f53bba|xn--hlcj6aya9esc7a|xn--jxalpdlp|xn--kgbechtv|xn--zckzah|ye|yt|yu|za|zm|zw)";
    this.PORT = "(:\\d+)?";
    this.HIER_PART = "(((\\/\\/)?((([\\w-.~!$&'()*+,;=:]|%[a-f\\d]{2})*@)?(" + this.IP + "|([\\w-.~!$&'()*+,;=]|%[a-f\\d]{2})*)" + this.PORT + "))([\\w-.~!$&'()*+,;=:@/]|%[a-f\\d]{2})*)";
//    this.PATH = "([;/]([\\w-.~!$&'()*+,;=:@/]|%[a-f\\d]{2})*)?";
    this.QUERY_FRAGMENT = "([&?]([\\w-.~!$&'()*+,;=:@/?]|%[a-f\\d]{2})*)?(#([\\w-.~!$&'()*+,;=:@/?]|%[a-f\\d]{2})*)?";

    this.URI1 = "(" + this.SCHEME + this.HIER_PART + this.QUERY_FRAGMENT + ")";
//    this.URI2 = "(" + this.AUTHORITY + "(" + this.IP + "|(" + this.REG_NAME + this.TLD + "\\b))" + this.PORT + this.PATH + this.QUERY_FRAGMENT + ")";

//    this.uriRegExp = new RegExp("^(" + this.URI1 + "|" + this.URI2 + ")$", "i");
    this.urlRegExp = new RegExp("^(" + this.URI1 + ")$", "i");
    this.domainRegExp = new RegExp("^(" + this.IP + "|(" + this.ULD + this.TLD + "\\b))" + this.PORT + "$", "i");

//    this.generateCSS = function()
//    {
//        var tmp =
//            this.elementBlacklist.generateCSS() +
//            this.styles.generateCSS();
//        if (tmp != "")
//        {
//            var mozDoc = "@-moz-document ";
//            if (this.domainRegExp.test(this.name)) mozDoc += "domain";
//            else if (this.uriRegExp.test(this.name)) mozDoc += "url-prefix";
//            else return "";
//            mozDoc += "(" + this.name + ")\n";
//            return mozDoc + "{\n" + tmp + "}\n\n";
//        }
//        else return "";
//    }

    this.generateCSS = function()
    {
        if (this.domainRegExp.test(this.name))
        {
            var tmp = this.elementBlacklist.generateCSS() +
                      this.styles.generateCSS();
            if (tmp != "") return "@-moz-document domain(" + this.name + ")\n{\n" + tmp + "}\n\n";
        }
        else
        {
            if (this.urlRegExp.test(this.name))
            {
                var res = "";
                var tmp = this.elementBlacklist.generateCSS(true) +
                          this.styles.generateCSS(true);
                if (tmp != "") res += "@-moz-document url(" + this.name + ")\n{\n" + tmp + "}\n\n";

                tmp = this.elementBlacklist.generateCSS() +
                      this.styles.generateCSS();
                if (tmp != "") res += "@-moz-document url-prefix(" + this.name + ")\n{\n" + tmp + "}\n\n";

                if (res != "") return res;
            }
        }

        return "";
    }

    this.setId(id);
    this.setName(name);
}
